"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const keymaster_1 = __importDefault(require("keymaster"));
exports.keymaster = keymaster_1.default;
const engine_1 = require("./engine");
exports.Cell = engine_1.Cell;
exports.GameEngine = engine_1.GameEngine;
const game_1 = require("./models/game");
exports.GameData = game_1.GameData;
const parser_1 = __importDefault(require("./parser/parser"));
exports.Parser = parser_1.default;
const sounds_1 = require("./sounds");
exports.closeSounds = sounds_1.closeSounds;
exports.playSound = sounds_1.playSound;
const base_1 = __importDefault(require("./ui/base"));
exports.BaseUI = base_1.default;
const table_1 = __importDefault(require("./ui/table"));
exports.TableUI = table_1.default;
const util_1 = require("./util");
exports.RULE_DIRECTION = util_1.RULE_DIRECTION;
class TableEngine {
    constructor(table, customHandler) {
        this.tableUI = new table_1.default(table);
        this.timer = 0;
        this.currentLevel = 0;
        const defaultEventHandler = {
            onSound: (sound) => {
                // let sounds play while the game loads or player keeps moving
                sounds_1.playSound(sound); // tslint:disable-line:no-floating-promises
                return;
            },
            onLevelComplete: () => {
                if (!this.tableUI.isCurrentLevelAMessage()) {
                    alert(`Congratulations! You completed the level.`);
                }
            },
            onMessage: (message) => alert(message),
            onWin: () => alert(`You WON!`)
        };
        this.eventHandler = {
            onSound: (customHandler ? customHandler.onSound : null) || defaultEventHandler.onSound,
            onLevelComplete: (customHandler ? customHandler.onLevelComplete : null) || defaultEventHandler.onLevelComplete,
            onMessage: (customHandler ? customHandler.onMessage : null) || defaultEventHandler.onMessage,
            onWin: (customHandler ? customHandler.onWin : null) || defaultEventHandler.onWin
        };
    }
    setGame(source, levelNum) {
        this.tableUI.setGame(source);
        this.tableUI.setLevel(levelNum);
        this.currentLevel = levelNum;
    }
    start() {
        this.startKeyboardListener();
        this.startTickHandler();
    }
    stop() {
        cancelAnimationFrame(this.timer);
    }
    startKeyboardListener() {
        keymaster_1.default('w', (e) => { e.preventDefault(); this.tableUI.pressUp(); });
        keymaster_1.default('s', (e) => { e.preventDefault(); this.tableUI.pressDown(); });
        keymaster_1.default('a', (e) => { e.preventDefault(); this.tableUI.pressLeft(); });
        keymaster_1.default('d', (e) => { e.preventDefault(); this.tableUI.pressRight(); });
        keymaster_1.default('x', (e) => { e.preventDefault(); this.tableUI.pressAction(); });
        keymaster_1.default('z, u', (e) => { e.preventDefault(); this.tableUI.pressUndo(); });
        keymaster_1.default('r', (e) => { e.preventDefault(); this.tableUI.pressRestart(); });
    }
    startTickHandler() {
        const runLoop = () => __awaiter(this, void 0, void 0, function* () {
            while (this.tableUI.isCurrentLevelAMessage()) {
                yield this.eventHandler.onMessage(this.tableUI.getCurrentLevelMessage());
                this.currentLevel++;
                this.tableUI.setLevel(this.currentLevel);
            }
            const { 
            // changedCells,
            didLevelChange, didWinGame, messageToShow, soundToPlay
            // wasAgainTick
             } = this.tableUI.tick();
            if (soundToPlay) {
                yield this.eventHandler.onSound(soundToPlay);
            }
            if (didWinGame) {
                yield this.eventHandler.onWin();
                cancelAnimationFrame(this.timer);
                return; // make sure we don't call window.requestAnimationFrame again
            }
            else if (didLevelChange) {
                this.currentLevel += 1;
                this.tableUI.setLevel(this.currentLevel);
                yield this.eventHandler.onLevelComplete(this.currentLevel);
            }
            else if (messageToShow) {
                yield this.eventHandler.onMessage(messageToShow);
                this.tableUI.pressAction(); // Tell the engine we are ready to continue
            }
            this.timer = window.requestAnimationFrame(runLoop);
        });
        this.timer = window.requestAnimationFrame(runLoop);
    }
}
exports.TableEngine = TableEngine;
//# sourceMappingURL=index-browser.js.map